/*!
 * zero-angularjs-helper - v1.0.84 - 2018-07-11
 * 
 * Copyright (c) 2018 - Mohos 'ZerosuxX' Tamás; Licensed MIT
 */
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as an anonymous module unless amdModuleId is set
    define([], function () {
      return (factory());
    });
  } else if (typeof module === 'object' && module.exports) {
    // Node. Does not work with strict CommonJS, but
    // only CommonJS-like environments that support module.exports,
    // like Node.
    module.exports = factory();
  } else {
    factory();
  }
}(this, function () {

angular.module('zerosuxx.zeroNgHelper', []);

angular.module('zerosuxx.zeroNgHelper') 
.directive('data', ['$rootScope', 'HttpClient', function($rootScope, HttpClient) {
    return {
        restrict: 'A',
        link: function(scope, element, attrs) {                
            scope.load = function(url, scopeAlias, queryParams) {
                HttpClient.get(url, {params: queryParams}).then(function(response) {
                    scope[scopeAlias ? scopeAlias : 'data'] = response.data;
                });
            };
            scope.reload = function() {
                scope.load(attrs.data, attrs.scopeAlias, attrs.params);
            };
            scope.reload();  
        }
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.directive('include', ['$templateUrl', function($templateUrl) {
    var generateId = function(type, src) {
        return type + '_' + src.replace(/-/g, '');
    };

    return {
        restrict: 'A',
        replace: true,
        link: function(scope, element, attrs) {
            var type = attrs.include;
            var src = attrs.src;
            if(type === 'element') {
                scope[generateId(type, src)] = $templateUrl.getElement(src);
            } else {
                scope[generateId(type, src)] = $templateUrl.getTemplate(type, src);
            }
        },
        template: function(element, attrs) {
            var onloadAttr = attrs.onload ? ' onload="'+attrs.onload+'"' : '';
            return '<div ng-include="' + generateId(attrs.include, attrs.src) + '"'+onloadAttr+'></div>';
        }
    };
}]);
angular.module('zerosuxx.zeroNgHelper') 
.directive('list', ['HttpClient', function(HttpClient) {
    var generatePages = function(count) {
        var pages = [];
        for (var i = 1; i <= count; i++) {
            pages.push(i);
        }
        return pages;
    };
    
    var resetPager = function(scope) {
        scope.prevPage = 1;
        scope.currentPage = 1;
        scope.nextPage = scope.currentPage+1;
    };

    return {
        restrict: 'A',
        link: function(scope, element, attrs) {
            var url = attrs.list;
            var scopeAlias = attrs.scopeAlias;
            var init = attrs.init;
            var maxPage = attrs.maxPage || 2;  
            var scrollTopOnPaginate = attrs.scrollTo || 0;  

            scope.onLoads = [];

            scope.queryParams = {};
            resetPager(scope);

            scope.paginate = function(page) {
                if(scope.currentPage === page || page < 1 || page > scope.lastPage) {
                    return;
                }
                if(page > 1) {
                    scope.prevPage = page-1;
                } else {
                    scope.prevPage = 1;
                }
                scope.currentPage = page;
                scope.slicePages(scope.currentPage, maxPage);
                if(scope.lastPage > page) {
                    scope.nextPage = page+1;
                }
                scope.load({page: page});
            };

            scope.initPages = function(pagesCount, reset) {
                scope.allPages = generatePages(pagesCount);
                scope.lastPage = parseInt(pagesCount);
                scope.slicePages(scope.currentPage, maxPage);
                if(reset) {
                    resetPager(scope);
                }
            };

            scope.setQueryParams = function(params) {
                scope.queryParams = params;
                return scope;
            };

            scope.slicePages = function(currentPage, maxPage) {
                var minPage = currentPage > maxPage ? currentPage - (maxPage+1) : 0;
                scope.pages = scope.allPages.slice(minPage, currentPage+maxPage);
            };

            scope.resetPager = resetPager;

            scope.load = function(queryParams) {
                if(!queryParams) {
                    queryParams = {};
                }
                angular.extend(queryParams, scope.queryParams);
                HttpClient.get(url, {params: queryParams}).then(function(response) {
                    var data = response.data;
                    scope[scopeAlias ? scopeAlias : 'list'] = data;
                    scope.initPages(data.pages_count, queryParams.page === undefined);
                    if(scope.onLoads.length > 0) {
                        for(var i in scope.onLoads) {
                            scope.onLoads[i](response);
                        }
                    }
                });
            };
            
            scope.onLoad = function(callback) {
                scope.onLoads.push(callback);
            };

            if(init === undefined || init !== '0') {
                scope.load();
            }
            
            if(scrollTopOnPaginate) {
                scope.onLoad(function() {
                     document.body.scrollTop = document.documentElement.scrollTop = 0;
                });
            }
        }
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.directive('ngSubmit', ['HttpClient', function(HttpClient) {
    return {
        restrict: 'A',
        link: function(scope, element, attrs) {
            var scrollTop = attrs.scrolltop;
            scope.validate = function(callback) {
                var errors = callback(scope, element[0], attrs);
                if(!(errors instanceof Object)) {
                    errors = {};
                }
                var hasError = Object.keys(errors).length > 0;
                if(hasError) {
                    scope.errors = errors;
                }
                return !hasError;
            };
            scope.resetForm = function(response, scope, element, attrs) {
                element.reset();
                return true;
            };
            scope.ajaxHandle = function(url, callback) {
                var method = attrs.method || 'POST';
                var args = arguments;
                HttpClient.request(method, url, element[0]).then(function(response) {
                    for(var i = 1; i < args.length; i++) {
                        if(typeof args[i] !== 'function') {
                            continue;
                        }
                        args[i](response, scope, element[0], attrs);
                    }
                    if(response.data.status && response.data.message) {
                        if(attrs.name) {
                            scope.alerts = {};
                            scope.alerts[attrs.name] = true;
                        }
                        scope.alert = {
                            status: response.data.status,
                            message: response.data.message
                        };
                        if(response.data.errors) {
                            scope.errors = response.data.errors;
                        } else {
                            scope.errors = [];
                        }
                        if(scrollTop) {
                            var scrollTo = 0;
                            if(scrollTop === 'element') {
                                scrollTo = element[0].offsetTop - element[0].scrollTop;
                            }
                            document.body.scrollTop = document.documentElement.scrollTop = scrollTo;
                        }
                    }
                });
                return true;
            };
            scope.handle = function(callback) {
                callback(scope, element[0], attrs);
                return true;
            };
        }
    };
}]);
angular.module('zerosuxx.zeroNgHelper')
.directive('clickHandler', ['HttpClient', function(HttpClient) {
    return {
        restrict: 'C',
        link: function(scope, element, attrs) {
            var method = attrs.method || 'GET';
            var params = attrs.params || {};
            var key = attrs.key;
            scope.ajaxHandle = function(url, callback) {
                HttpClient.request(method, url, params).then(function(response) {
                    var enable = !scope.$eval(key);
                    if(callback) {
                        callback(enable, scope, element[0], attrs);
                    }
                    if(response.data.status === 'success') { 
                        scope.$eval(key + ' = ' + enable);
                    }
                });
            };
            
            scope.handle = function(callback) {
                var enable = !scope.$eval(key);
                callback(enable, scope, element[0], attrs);
                return true;
            };
        }
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.factory('DisableReloadCurrentTemplate', ['$route', function($route) {
    return function(scope) {
        var lastRoute = $route.current;
        var remove = scope.$on('$locationChangeSuccess', function() {
            if($route.current.$$route && lastRoute.$$route.templateUrl === $route.current.$$route.templateUrl) {
                $route.current = lastRoute;
                remove();
            }
        });
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.factory('HttpApiInterceptor', ['$location', '$q', '$zeroConfig', 'StorageService', function HttpApiInterceptor($location, $q, $zeroConfig, StorageService) {
    var config = $zeroConfig.config.httpApiInterceptors;
    return {
        responseError: function(response) {
            if(response.status === 401) {
                if(config.unauthorized.savePreviousLocationKey) {
                    StorageService.set(config.unauthorized.savePreviousLocationKey, $location.path());
                }
                $location.path(config.locations.unauthorized).search({});
            } else if(response.status === 404) {
                $location.path(config.locations.notFound).search({});
            }
            return $q.reject(response);
        }
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.filter('sceTrustAs', ['$sce', function($sce) {
    return function(input, type) {
        return typeof input === 'string' ? $sce.trustAs(type || 'html', input) : '';
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.provider('$templateUrl', function $templateUrlProvider() {
    this.templateDir = 'templates';
    this.suffix = '';

    this.$get = function() {
        return this;
    };

    this.setTemplateDir = function(dir) {
        this.templateDir = dir;
    };

    this.setSuffix = function(suffix) {
        this.suffix = suffix;
    };

    this.get = function(templateData) {
        var data = {
            templateName: 'index',
            ext: 'html'
        };
        if(angular.isArray(templateData)) {
            data.dir = templateData[0];
            if(templateData[1]) {
                data.templateName = templateData[1];
            }
            if(templateData[2]) {
                data.ext = templateData[2];
            }
        } else {
            angular.extend(data, templateData);
        }
        var templateFile = this.templateDir + '/' + data.dir + '/' + data.templateName + '.' + data.ext + this.suffix;
        return templateFile;
    };

    this.getElement = function(templateName, ext) {
        return this.get(['_elements', templateName, ext]);
    };
	
	this.getTemplate = function(dir, templateName, ext) {
        return this.get([dir, templateName, ext]);
    };
});

angular.module('zerosuxx.zeroNgHelper') 
.provider('$zeroRoute', function $zeroRouteProvider() {
    this.templateUrlResolver = null;
    this.routeProvider = null;

    this.$get = function() {
        return this;
    };

    this.setTemplateUrlResolver = function(resolver) {
        this.templateUrlResolver = resolver;
        return this;
    };
    
    this.setRouteProvider = function(routeProvider) {
        this.routeProvider = routeProvider;
        angular.extend(this, routeProvider);
        return this;
    };

    this.connect = function(route, template, controller, extendedConfig) {
        if(!this.templateUrlResolver) {
            throw new Error('TemplateResolver not found!');
        }
        if(!this.routeProvider) {
            throw new Error('RouteProvider not found!');
        }
        var config = {};
        if(extendedConfig === undefined) {
            extendedConfig = {};
        }
        var templateData = angular.isArray(template) ? template : [template];

        config.templateUrl = this.templateUrlResolver(templateData);
        if(controller) {
            config.controller = controller;
            if(extendedConfig.controllerAs === undefined) {
                extendedConfig.controllerAs = 'ctrl';
            }
        }
        if(extendedConfig) {
            angular.extend(config, extendedConfig);
        }
        return this.when(route, config);
    };
});

angular.module('zerosuxx.zeroNgHelper') 
.provider('$zeroConfig', ['$templateUrlProvider', '$locationProvider', '$logProvider', '$httpProvider', '$qProvider', 
function $zeroConfigProvider($templateUrlProvider, $locationProvider, $logProvider, $httpProvider, $qProvider) {
    this.defaults = {
        app: {
            debug: true,
            version: 'DEV',
            versionCache: true
        },
        template: {
            templateDir: 'templates',
            suffix: ''
        },
        translateStaticFiles: {
            prefix: 'locales/',
            suffix: '.json'
        },
        route: {
            provider: null,
            zeroProvider: null,
            connects: [],
            otherwise: ''
        },
        locationHtml5: {
            enabled: true,
			requireBase: false
        },
        http: {
            interceptors: []
        },
        httpApiInterceptors: {
            locations: {
                unauthorized: '/login',
                notFound: '/404'
            },
            unauthorized: {
                savePreviousLocationKey: 'loginRedirect'
            }
        }
    };
    this.config = this.defaults;
    this.zeroRouteProvider = null;
    this.translateProvider = null;
        
    this.$get = function() {
        return this;
    };
    
    this.setConfig = function(config) {
        var keys = Object.keys(this.defaults);
        for(var i in keys) {
            var key = keys[i];
            var defaultConfig = this.defaults[key];
            if(!config[key]) {
                config[key] = {};
            }
            this.config[key] = angular.extend(this.config[key], defaultConfig, config[key]);
        }
        return this;
    };
    
    this.setZeroRouteProvider = function(zeroRouteProvider) {
        this.zeroRouteProvider = zeroRouteProvider;
        return this;
    };
    
    this.setTranslateProvider = function(translateProvider) {
        this.translateProvider = translateProvider;
        return this;
    };
    
    this.load = function() {
        this.configureDebugMode();
        
        this.configureLocation();
        
        this.configureHttp();
        
        this.configureTranslate();
        
        this.configureTemplateResolver();
        
        this.configureRoute();
    };
    
    this.configureDebugMode = function() {
        var debug = this.config.app.debug;
        $logProvider.debugEnabled(debug);
        $qProvider.errorOnUnhandledRejections(debug);
        return this;
    };
    
    this.configureLocation = function() {
        $locationProvider.html5Mode(this.config.locationHtml5);
        return this;
    };
    
    this.configureHttp = function() {
        if(this.config.http.interceptors.length) {
            for(var i in this.config.http.interceptors) {
                $httpProvider.interceptors.push(this.config.http.interceptors[i]);
            }
        }
        return this;
    };
    
    this.configureTranslate = function() {
        if(this.translateProvider) {
            var configStaticFiles = this.config.translateStaticFiles;
            if(configStaticFiles) {
                if(this.config.app.versionCache) {
                    configStaticFiles.suffix += '?v=' + this.config.app.version;
                }
                this.translateProvider.useStaticFilesLoader(configStaticFiles);
                //this.translateProvider.useSanitizeValueStrategy('escape');
            }
        }
        return this;
    };
    
    this.configureTemplateResolver = function() {
        var config = this.config.template;
        $templateUrlProvider.setTemplateDir(config.templateDir);
        var suffix = '';
        if(config.suffix) {
            suffix = config.suffix;
        }
        if(this.config.app.versionCache) {
            suffix += '?v=' + this.config.app.version;
        }
        $templateUrlProvider.setSuffix(suffix);
        return this;
    };
    
    this.configureRoute = function() {
        var config = this.config.route;
        if(config.zeroProvider) {
            this.setZeroRouteProvider(config.zeroProvider);
        }
        if(this.zeroRouteProvider && config.connects) {
            if(config.provider) {
                this.zeroRouteProvider.setRouteProvider(config.provider);
            }
            if(this.zeroRouteProvider.templateUrlResolver === null) {
                this.zeroRouteProvider.setTemplateUrlResolver(function(data) {
                    return $templateUrlProvider.get(data);
                });
            }
            this.loadRoutes(config, this.zeroRouteProvider);
        }
        return this;
    };
    
    this.loadRoutes = function(routeConfig, zeroRouteProvider) {
        for(var i in routeConfig.connects) {
            var data = routeConfig.connects[i];
            zeroRouteProvider.connect(data[0], data[1], data[2], data[3]);
        }
        if(routeConfig.otherwise) {
            zeroRouteProvider.otherwise(routeConfig.otherwise);
        }
        return this;
    };
}]);

angular.module('zerosuxx.zeroNgHelper') 
.service('Auth', ['HttpClient', 'StorageService', function Auth(HttpClient, StorageService) {
    this.init = function() {
        var auth = StorageService.getUnSerialized('Auth');
        if(auth) {
            HttpClient.defaults.headers.common['Auth-Token'] = auth.token;
        }
    };    

    this.login = function(data) {
        StorageService.setSerialized('Auth', data);
        this.init();
    };

    this.getUser = function(key) {
        var data = StorageService.getUnSerialized('Auth');
        if(key !== undefined) {
            data = data[key];
        }
        return data;
    };

    this.logout = function() {
        StorageService.remove('Auth');
        HttpClient.defaults.headers.common['Auth-Token'] = undefined;
    };

    this.isLogged = function() {
        return this.getUser() !== null;
    };
}]);


angular.module('zerosuxx.zeroNgHelper') 
.service('EventDispatcher', function EventDispatcher() {
    this.dispatch = function(name, data) {
        var evt = document.createEvent('Event');  
        evt.data = data;
        evt.initEvent(name, false, false);  
        window.dispatchEvent(evt);
        return evt;
    };
});



angular.module('zerosuxx.zeroNgHelper')
.service('HttpClient', ['$http', function HttpClient($http) {
    angular.extend(this, $http);

    this.request = function(type, url, data, config) {
        if(config === undefined) {
            config = {};
        }
        var lowerType = type.toLowerCase();
        if(lowerType === 'get') {
            return this.getWithData(url, data, config);
        } else if(lowerType === 'post') {
            return this.postFormData(url, data, config);
        }
    };

    this.getWithData = function(urlOrConfig, data, config) {
        if(config === undefined) {
            config = {};
        }
        if(data instanceof Element) {
            config.paramSerializer = function(params) {
                return new URLSearchParams(params).toString();
            };
            config.params = new FormData(data);
        } else {
            config.params = data;
        }
        return this.get(urlOrConfig, config);
    };

    this.postFormData = function(urlOrConfig, data, config) {
        var formElement = data instanceof Element ? data : undefined;

        var formData = new FormData(formElement);
        if(formElement === null) {
            for(var key in Object.keys(data)) {
                formData.append(key, data[key]);
            }
        }
        if(config === undefined) {
            config = {};
        }
        if(typeof config.headers === 'undefined') {
            config.headers = {};
            config.headers['Content-Type'] = undefined;
        }
        return this.post(urlOrConfig, formData, config);
    };
}]);

angular.module('zerosuxx.zeroNgHelper')
.service('StorageService', function StorageService() {
    this.storage = localStorage;

    this.get = function(key) {
        return this.storage.getItem(key);
    };

    this.getUnSerialized = function(key) {
        var value = this.get(key);
        return value ? JSON.parse(this.get(key)) : null;
    };

    this.consume = function(key) {
        var value = this.get(key);
        this.remove(key);
        return value;
    };

    this.getKey = function(index) {
        return this.storage.key(index);
    };

    this.set = function(key, value) {
        return this.storage.setItem(key, value);
    };

    this.setSerialized = function(key, value) {
        return this.set(key, JSON.stringify(value));
    };

    this.remove = function(key) {
        return this.storage.removeItem(key);
    };

    this.count = function() {
        return this.storage.length;
    };

    this.length = function() {
        return this.count();
    };

    this.getStorage = function(storage) {
        return this.storage;
    };

    this.setStorage = function(storage) {
        this.storage = storage;
        return this;
    };
});
return 'zerosuxx.zeroNgHelper';

}));
